package envoy.server.processors;

import java.io.IOException;
import java.util.Date;

import envoy.data.Message;
import envoy.event.MessageStatusChangeEvent;
import envoy.server.ConnectionManager;
import envoy.server.ObjectProcessor;
import envoy.server.database.PersistenceManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * This {@link ObjectProcessor} handles incoming {@link Message}s.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>MessageProcessor.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Server Standalone v0.1-alpha
 */
public class MessageProcessor implements ObjectProcessor<Message> {

	@Override
	public Class<Message> getInputClass() { return Message.class; }

	@Override
	public void process(Message message, long socketId, ObjectWriteProxy writeProxy) {

		ConnectionManager connectionManager = ConnectionManager.getInstance();
		message.nextStatus();
		if (connectionManager.isOnline(message.getRecipientId())) try {
			// If recipient is online, send the message directly
			writeProxy.write(connectionManager.getSocketId(message.getRecipientId()), message);

			// Update the message status to RECEIVED
			message.setReceivedDate(new Date());
			message.nextStatus();
			writeProxy.write(connectionManager.getSocketId(message.getSenderId()), new MessageStatusChangeEvent(message));
		} catch (IOException e) {
			System.err.println("Recipient online. Failed to send message" + message.getId());
			e.printStackTrace();
		}
		PersistenceManager.getPersistenceManager().addMessage(new envoy.server.data.Message(message));
	}
}
